<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

require_once(__DIR__ . '/vendor/autoload.php');


use Resursbank\RBEcomPHP\RESURS_CALLBACK_TYPES;
use Resursbank\RBEcomPHP\RESURS_ENVIRONMENTS;
use Resursbank\RBEcomPHP\RESURS_FLOW_TYPES;
use Resursbank\RBEcomPHP\RESURS_PAYMENT_STATUS_RETURNCODES;
use Resursbank\RBEcomPHP\RESURS_URL_ENCODE_TYPES;

define('OMNICHECKOUTCREATE', 'omniCheckoutCreate');
define('OMNIREF', 'omniRef');
/**
 * Resurs bank Payments for Woocommerce.
 *
 * @class 		Invoice_Pay
 * @extends		WC_Payment_Gateway
 * @version		1.0.0
 */
class Invoice_Pay extends WC_Payment_Gateway {

    private $auth;
    private $validCred;
    private $resursPayMethod;
    private $omni_vars;
    private $omniFallBacks;
    private $woo_session_valid;
    private $payId;
    private $baseURL;
    private $callback_types;
    private $cartID;
    private $sandbox;//To access the out side woocommerce functions
    private $log_enabled;//This is access log function outside woocommerce functions

	private $logger;
	private $logger_context;
	private $merchantID;
	private $password;

	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->id                 = 'invoice_pays';
		$this->method_title       = __( 'MoreFlo Checkout', MOREFLO_CHECKOUT );
        $this->method_description = __( 'Use invoice, Swish and partial payment and validate the customers credibility with Resurs Bank. Requires SSL when in live mode.', MOREFLO_CHECKOUT );
        $this->has_fields         = true;
        $this->supports           = array(
            'products',
            'tokenization',
        );

        // Initialize the logger
        $this->logger = wc_get_logger();
        $this->logger_context = array( 'source' => $this->id );

        $this->callback_types = $this->getCallbackTypes();

		// Load the form fields
		$this->init_form_fields();

		// Load the settings.
        $this->init_settings();
        $this->cartID = url_to_postid(wc_get_checkout_url());


		// Get setting values
        $this->enabled         = $this->get_option( 'enabled' );
        $this->sandbox         = $this->get_option( 'sandbox' );
        $this->title           = $this->get_option( 'title' );
		$this->description     = $this->get_option( 'description' );
		$this->merchantID      = $this->sandbox == 'no' ? $this->get_option( 'live_merchant_id' ) : $this->get_option( 'sandbox_merchant_id' );
        $this->password        = $this->sandbox == 'no' ? $this->get_option( 'live_password' ) : $this->get_option( 'sandbox_password' );
        $this->log_enabled     = $this->get_option( 'log_enabled' );

        $this->authentication();
        // initial session setup.
        $this->validate_session();
        $this->init_session_variables();
        $this->set_base_url();
        // $this->setEnviromentBaseSelect();

        // General Hooks
        add_action('woocommerce_api_invoice_pay', [$this, 'resurs_callback_response']);

        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            // add the action
        add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ), 1, 0 );
        add_action( 'woocommerce_after_checkout_form', array($this, 'resurs_omnicheckout_form_location'));//Iframe form loads
    }

    public function is_available() {
        if(class_exists('WC_Subscriptions_Cart'))
        {
            if ( WC_Subscriptions_Cart::cart_contains_subscription() )
            {
                return false;
            }
        }
        return parent::is_available();
	}
    /**
     * Resurs payment put method if cart updated this function trigger
     */
    public static function resurs_omnicheckout_update()
    {
        $payoption = get_option('woocommerce_invoice_pays_settings');

        if(isset($payoption['enabled']) && 'no' != $payoption['enabled'])
        {
            if(is_checkout() && (WC()->session->get(OMNICHECKOUTCREATE) == 'yes') && WC()->cart->get_cart_contents_count() >= 1 && !empty(WC()->session->get(OMNIREF)))
            {
                $currentOmniRef = WC()->session->get(OMNIREF);
                $payoption = get_option('woocommerce_invoice_pays_settings');

                $user      = $payoption['sandbox'] == 'no' ? $payoption['live_merchant_id'] : $payoption['sandbox_merchant_id'];
                $pw        = $payoption['sandbox'] == 'no' ? $payoption['live_password'] : $payoption['sandbox_password'];

                $auth;
                if($user && $pw)
                {
                    // $auth = new \Resursbank\RBEcomPHP\ResursBank($user, $pw);
                    if ($payoption['sandbox'] == "yes") {
                        $auth = new \Resursbank\RBEcomPHP\ResursBank($user, $pw, RESURS_ENVIRONMENTS::ENVIRONMENT_TEST);
                    }else{
                        $auth = new \Resursbank\RBEcomPHP\ResursBank($user, $pw, RESURS_ENVIRONMENTS::ENVIRONMENT_PRODUCTION);
                    }
                }
                $paymentSpec = self::get_payment_spec(WC()->cart);
                $result = $auth->updateCheckoutOrderLines($currentOmniRef, $paymentSpec['specLines']);
            }
        }
    }
    /**
     * Admin field validatation
     */
    function process_admin_options(){
        parent::process_admin_options();
        if ( !empty( $_POST['woocommerce_invoice_pays_sandbox'] ) && (empty( $_POST['woocommerce_invoice_pays_sandbox_merchant_id'] ) ||  empty( $_POST['woocommerce_invoice_pays_sandbox_password'] )) ) {
            WC_Admin_Settings::add_error( 'Error: Please fill sandbox fields' );
            return false;
        }
        if ( empty( $_POST['woocommerce_invoice_pays_sandbox'] ) && (empty( $_POST['woocommerce_invoice_pays_live_merchant_id'] ) ||  empty( $_POST['woocommerce_invoice_pays_live_password'] ))) {
            WC_Admin_Settings::add_error( 'Error: Please fill live fields' );
            return false;
        }
    }
	/**
	 * Initialise Gateway Settings Form Fields.
	 */
	public function init_form_fields() {

        $label = __( 'Enable Logging', MOREFLO_CHECKOUT );
        $description = __( 'Enable the logging of errors.', MOREFLO_CHECKOUT );

        if ( defined( 'WC_LOG_DIR' ) ) {
            $invoice_log_url = add_query_arg( 'tab', 'logs', add_query_arg( 'page', 'wc-status', admin_url( 'admin.php' ) ) );
            $invoice_log_key = 'invoice_pays-' . sanitize_file_name( wp_hash( 'invoice_pays' ) ) . '-log';
            $invoice_log_url = add_query_arg( 'log_file', $invoice_log_key, $invoice_log_url );

            $label .= ' | ' . sprintf( __( '%1$sView Log%2$s', MOREFLO_CHECKOUT ), '<a href="' . esc_url( $invoice_log_url ) . '">', '</a>' );
        }

		$this->form_fields = array(
            'enabled' => array(
				'title'       => __( 'Enable/Disable', MOREFLO_CHECKOUT ),
				'label'       => __( 'Enable payment using Resurs Bank payment gateway', MOREFLO_CHECKOUT ),
				'type'        => 'checkbox',
				'description' => '',
                'default'     => 'no'
            ),
            'sandbox' => array(
				'title'       => __( 'Sandbox', MOREFLO_CHECKOUT ),
				'label'       => __( 'Enable Sandbox Mode', MOREFLO_CHECKOUT ),
				'type'        => 'checkbox',
				'description' => __( 'Enabling test mode(real payments will not be taken).', MOREFLO_CHECKOUT ),
				'default'     => 'yes'
            ),
			'title' => array(
				'title'       => __( 'Title', MOREFLO_CHECKOUT ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout within the credit card payment type.', MOREFLO_CHECKOUT ),
				'default'     => __( 'Pay with Resurs Bank', MOREFLO_CHECKOUT )
			),
			'description' => array(
				'title'       => __( 'Description', MOREFLO_CHECKOUT ),
				'type'        => 'text',
				'description' => __( 'This controls the description which the user sees during checkout within the credit card payment type.', MOREFLO_CHECKOUT ),
				'default'     => __( 'Pay with Resurs Bank', MOREFLO_CHECKOUT )
			),
            'sandbox_merchant_id' => array(
				'title'       => __( 'Sandbox - Merchant ID', MOREFLO_CHECKOUT ),
				'type'        => 'text',
				'description' => __( 'This is the sandbox account merchant ID', MOREFLO_CHECKOUT ),
				'default'     => ''
			),
			'sandbox_password' => array(
				'title'       => __( 'Sandbox password', MOREFLO_CHECKOUT ),
				'type'        => 'text',
				'description' => __( 'This is the sandbox password. Follow these instructions create accounts https://tinyurl.com/waoxrnq.', MOREFLO_CHECKOUT ),
				'default'     => ''
            ),
			'live_merchant_id' => array(
				'title'       => __( 'Live - Merchant ID', MOREFLO_CHECKOUT ),
				'type'        => 'text',
				'description' => __( 'This is the live account merchant ID .', MOREFLO_CHECKOUT ),
				'default'     => ''
			),
			'live_password' => array(
				'title'       => __( 'Live password', MOREFLO_CHECKOUT ),
				'type'        => 'text',
				'description' => __( 'This is the live password. Follow these instructions create accounts https://tinyurl.com/waoxrnq.', MOREFLO_CHECKOUT ),
				'default'     => ''
            ),
            'log_enabled' => array(
                'title'       => __( 'Debug Log', MOREFLO_CHECKOUT ),
                'label'       => $label,
                'description' => $description,
                'type'        => 'checkbox',
                'default'     => 'no'
            )
        );
    }


    public function payment_scripts() {

        // if our payment gateway is disabled, we do not have to enqueue JS too
        if ( 'no' === $this->enabled ) {
            return;
        }

        // no reason to enqueue JavaScript if API keys are not set
        if ( empty( $this->merchantID ) || empty( $this->password ) ) {
            return;
        }
        wp_enqueue_script('resursomni', plugin_dir_url(__FILE__) . 'js/resurscheckout.js', array(),'', "");

        $this->localize_variable();
        $this->localize_variable_fallback();
        $ajaxObject = ['ajax_url' => admin_url('admin-ajax.php')];
        $transArray = array(
            'payError' => __( 'There is error on payment, please try again later', MOREFLO_CHECKOUT ),
        );

        wp_enqueue_script(
            'moreflo-js',
            plugin_dir_url( __FILE__ ) .'js/moreflo_invoice_pay.js',
            array( 'jquery' ),
            filemtime( plugin_dir_path( __FILE__ ) .'js/moreflo_invoice_pay.js' ),
            true );
        wp_enqueue_script(
            'moreflo-js-custom',
            plugin_dir_url( __FILE__ ) .'js/moreflo_invoice_pay_customs.js',
            array( 'jquery' ),
            filemtime( plugin_dir_path( __FILE__ ) .'js/moreflo_invoice_pay_customs.js' ),
            true );
        wp_enqueue_style(
            'moreflow_pay_css',
            plugin_dir_url( __FILE__ ) .'css/moreflow_Pay.css',
            array(),
            filemtime( plugin_dir_path( __FILE__ ) .'css/moreflow_Pay.css' ),
            'all' );

        $OmniVars = $this->omni_vars;
        $omniFail = $this->omniFallBacks;
        wp_localize_script('moreflo-js', 'ajax_object', $ajaxObject);
        wp_localize_script('moreflo-js-custom', 'ajax_object', $ajaxObject);
        wp_localize_script('moreflo-js', 'Omnivars', $OmniVars);
        wp_localize_script('moreflo-js', 'failback', $omniFail);
        wp_localize_script('moreflo-js', 'transArray', $transArray);

    }

    public function localize_variable()
    {
        $omniBookUrl = get_site_url().'/';
         $omniBookNonce = add_query_arg('wc-ajax', 'checkout', $omniBookUrl);

         if ($this->sandbox == "yes") {
            $OmniUrl = $this->auth->getCheckoutUrl(RESURS_ENVIRONMENTS::ENVIRONMENT_TEST);
        }else{
            $OmniUrl = $this->auth->getCheckoutUrl(RESURS_ENVIRONMENTS::ENVIRONMENT_PRODUCTION);
        }

        if ($this->woo_session_valid) {
            $omniRef = WC()->session->get(OMNIREF);
            $omniRefCreated = WC()->session->get('omniRefCreated');
            $omniRefAge = intval(WC()->session->get('omniRefAge'));
        }

        $gateways = WC()->payment_gateways()->get_available_payment_gateways();

        $OmniVars = [
            'RESURSCHECKOUT_IFRAME_URL' => $OmniUrl,
            'ACCEPT_CHECKOUT_PREFIXES' => '',
            // 'RESURSCHECKOUT' => home_url(),
            'RESURSCHECKOUT' => get_site_url().'/',
            'OmniPreBookUrl' => $omniBookNonce,
            //'OmniPreBookUrl' => $omniBookNonce,
            'OmniRef' => isset($omniRef) && !empty($omniRef) ? $omniRef : null,
            'OmniRefCreated' => isset($omniRefCreated) && !empty($omniRefCreated) ? $omniRefCreated : null,
            'OmniRefAge' => $omniRefAge,
            'isResursTest' => $this->sandbox,
            'gatewayCount' => (is_array($gateways) ? count($gateways) : 0),
            'postidreference' => '1'
        ];

        $this->omni_vars = $OmniVars;

    }

    private function localize_variable_fallback()
    {
        $this->omniFallBacksUrl = get_site_url().'/';

        $omniRef = WC()->session->get(OMNIREF);

        $this->omniFallBacksUrl = add_query_arg('wc-api', 'Invoice_Pay', $this->omniFallBacksUrl);
        $this->omniFallBacksUrl = add_query_arg('event-type', 'fallPayment', $this->omniFallBacksUrl);
        $this->omniFallBacksUrl = add_query_arg('payment_id', $omniRef, $this->omniFallBacksUrl);
        $this->omniFallBacksUrl = add_query_arg('omnicheckout_callback_mode', wp_create_nonce('omnicheckout_callback_nonce'), $this->omniFallBacksUrl);

        $this->omniFallBacks = $this->omniFallBacksUrl;
    }

    public function validate_session()
    {
        if (isset(WC()->session)) {
            $this->woo_session_valid = true;
        }
    }

    public function init_session_variables()
    {
        if ($this->woo_session_valid) {

            $omniRef = $this->payId;

            $omniRefCreated = WC()->session->get('omniRefCreated');
            $currentOmniRefAge = time() - $omniRefCreated;

            $currentOmniRef = WC()->session->get(OMNIREF);
            if (empty($currentOmniRef)) {
                /*
                * Empty references, create
                */

                WC()->session->set(OMNIREF, $omniRef);
                WC()->session->set('omniRefCreated', time());
                WC()->session->set('omniRefAge', $currentOmniRefAge);
            }
        }
    }
    /**
     * Load the Resurs form
     */
    public function resurs_omnicheckout_form_location()
    {
        if ( 'no' === $this->enabled ) {
            return;
        }

        // // no reason to enqueue JavaScript if API keys are not set
        if ( empty( $this->merchantID ) || empty( $this->password ) ) {
            return;
        }

        // Prepare the frame
        try {
            $frameDisplay = '<div id="resurs-checkout-container" style="display:none;">' . $this->resurs_omnicheckout_create_frame() ."</div>";
        } catch (Exception $e) {
            $frameContent = __(
                'We are unable to load Resurs Checkout for the moment. Please try again later.',
                MOREFLO_CHECKOUT
            );
            $frameDisplay = '<div class="col2-set label-warning" style="border:1px solid red; text-align: center;" id="resurs-checkout-container">' . $frameContent . "<!-- \n" . $e->getMessage() . " --></div>";
            $this->resetSession();
        }
        echo $frameDisplay;
    }

    public function retrieveEnviroment()
    {
        return $this->auth->getEnvironment();
    }

    public function setEnviromentBaseSelect()
    {
        if ($this->sandbox == "yes") {
            $this->auth->setEnvironment(RESURS_ENVIRONMENTS::ENVIRONMENT_TEST);
        }else{
            $this->auth->setEnvironment(RESURS_ENVIRONMENTS::ENVIRONMENT_PRODUCTION);
        }
    }

    function set_base_url()
    {
        if ($this->sandbox == "yes") {
            $baseTestURL = 'https://ecommerce.resurs.com/ws/V4/';
        }else{
            $baseTestURL = 'https://test.resurs.com/ecommerce-test/ws/V4/';
        }
        $this->baseURL = $baseTestURL;
    }

    public function authentication()
    {
        $user = $this->merchantID;
        $pw = $this->password;

        if($user && $pw && 'no' != $this->enabled){
            if ($this->sandbox == "yes") {
                $this->auth = new \Resursbank\RBEcomPHP\ResursBank($user, $pw, RESURS_ENVIRONMENTS::ENVIRONMENT_TEST);
            }else{
                $this->auth = new \Resursbank\RBEcomPHP\ResursBank($user, $pw, RESURS_ENVIRONMENTS::ENVIRONMENT_PRODUCTION);
            }
            $this->auth->setWsdlCache(true);
            $this->validateCred();
            if($this->validCred){
                $this->setEnviromentBaseSelect();
                $this->createPaymentID();
                $this->updateRegisterCallbacks();
            }
            else
            {
               WC_Admin_Settings::add_error(  __( 'Credential are incorrect, Please verify if you entered correct credential details', MOREFLO_CHECKOUT ) );
            }

        }
    }

    public function getpayMethodTrigger()
    {
        $this->getPaymentMethodsCus();
    }

    public function validateCred()
    {
        $this->validCred = $this->auth->validateCredentials();
    }

    public function createPaymentID()
    {
        $this->payId = $this->auth->getPreferredPaymentId();
    }

    public function getPaymentMethodsCus()
    {
        if($this->validCred){
            $this->resursPayMethod = $this->auth->getPaymentMethods();
        }
    }

    protected function resurs_omnicheckout_create_frame()
    {
        global $post;

        $this->auth->setPreferredPaymentFlowService(RESURS_FLOW_TYPES::RESURS_CHECKOUT);
        $bookDataOmni = self::createResursOmniOrder();// Create backend order for Iframe from cart details Which will create payment id based on that
        $this->write_Woo_logs('createResursOmniOrder object', print_r($bookDataOmni, true));

        $omniRef = WC()->session->get(OMNIREF);
        $omniRefCreated = WC()->session->get('omniRefCreated');

        //$this->auth->setWaitForFraudControl(true);//This is stop getting frozen automatically

        try {
            $customerId = self::getResursWooCustomerId();

            if (!is_null($customerId)) {
                $this->auth->setMetaData('CustomerId', $customerId);
            }

            $orderReferenceResults = $this->auth->createPayment($omniRef, $bookDataOmni);//If success payment created it will be generate Iframe and else if will false
            $resultIframe = is_string($orderReferenceResults) ? $orderReferenceResults : "";//This is similar output as this https://test.resurs.com/docs/display/ecom/Resurs+Checkout#ResursCheckout-/payments/{orderReference}-POST

            /**
             * Need to save paymentSessionID id when iframe button click future
             * Incase if user browser closed need to have reference
             */

            $resultIframe .= '<noscript><b>' .
                __(
                    'Resurs Checkout will not work properly without Javascript functions enabled',
                    MOREFLO_CHECKOUT
                )
                . '</b></noscript>';
                WC()->session->set(OMNICHECKOUTCREATE, 'yes');
        } catch (Exception $e) {
            $errorUnable = __(
                'We are unable to load Resurs Checkout for the moment. Please try again later.',
                MOREFLO_CHECKOUT
            );
            $resultIframe = '<div class="col2-set label-warning" style="border:1px solid red; text-align: center;" id="resurs-checkout-container">' . $errorUnable . "<!-- \n" . $e->getMessage() . " --></div>";
            $this->write_Woo_logs('Error details from payment gateway', $e->getMessage());
            $this->resetSession();
        }

        return $resultIframe;
    }

    public function resetSession()
    {
        WC()->session->set(OMNIREF, null);
        WC()->session->set('omniId', null);
        WC()->session->set('omniRefCreated', null);
        WC()->session->set('omniRefAge', null);
        WC()->session->set(OMNICHECKOUTCREATE, null);
    }

    private function getDataFromCustomerObject($billingObject, $key)
    {
        $return = '';
        if (isset($billingObject[$key])) {
            $return = $billingObject[$key];
        }
        return $return;
    }
    /**
     * create backend order for Iframe from cart details
     * Which will create payment id based on that
     */
    private function createResursOmniOrder()
    {
        global $woocommerce;
        $specLines = self::get_payment_spec($woocommerce->cart, true);

        $getUrls = $this->createReturnUrls();

        $bookDataOmni = [
            'orderLines' => $specLines,
        ];

        // If user is logged in, fetch data and send to RCO.
        // Note: Cookies may override this feature.
        if (is_user_logged_in()) {
            $this->auth->setFlag('KEEP_RCO_DELIVERY', true);
            $loggedInCustomer = new WC_Customer(get_current_user_id());
            $loggedInBilling = $loggedInCustomer->get_billing();
            $this->auth->setCustomer(
                '',
                '',
                $this->getDataFromCustomerObject($loggedInBilling, 'phone'),
                $this->getDataFromCustomerObject($loggedInBilling, 'email'),
                'NATURAL',
                ''
            );
            if(!empty($loggedInBilling['first_name']) && !empty($loggedInBilling['last_name']) && !empty($loggedInBilling['address_1']) && !empty($loggedInBilling['country'])){
            $this->auth->setDeliveryAddress(
                $this->getDataFromCustomerObject(
                    $loggedInBilling,
                    'first_name'
                ) . ' ' . $this->getDataFromCustomerObject(
                    $loggedInBilling, 'last_name'
                ),
                $this->getDataFromCustomerObject($loggedInBilling, 'first_name'),
                $this->getDataFromCustomerObject($loggedInBilling, 'last_name'),
                $this->getDataFromCustomerObject($loggedInBilling, 'address_1'),
                $this->getDataFromCustomerObject($loggedInBilling, 'address_2'),
                $this->getDataFromCustomerObject($loggedInBilling, 'city'),
                $this->getDataFromCustomerObject($loggedInBilling, 'postcode'),
                $this->getDataFromCustomerObject($loggedInBilling, 'country')
            );
            }
        }

        $responseforsingin = $this->auth->setSigning(
            $getUrls['successUrl'],
            $getUrls['backUrl'],
            false,
            $getUrls['backUrl'],
            RESURS_URL_ENCODE_TYPES::NONE
        );

        $shopUrl;
        if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on')
        {
            $shopUrl = "https";
        }
        else
        {
            $shopUrl = "http";
        }

        // Here append the common URL characters.
        $shopUrl .= "://";

        // Append the host(domain name, ip) to the URL.
        $shopUrl .= $_SERVER['HTTP_HOST'];

        $bookDataOmni['shopUrl'] = $shopUrl;

        return $bookDataOmni;
    }

    function getResursWooCustomerId($order = null)
    {
        $return = null;

        if (function_exists('wp_get_current_user')) {
            $current_user = wp_get_current_user();
        } else {
            $current_user = get_currentuserinfo();
        }

        if (isset($current_user->ID)) {
            $return = $current_user->ID;
        }

        // Created orders has higher priority since this id might have been created during order processing
        if (!is_null($order)) {
            $return = $order->get_user_id();
        }

        return $return;
    }

    protected static function get_payment_spec($cart, $specLinesOnly = false)
    {
        global $woocommerce;

        $shippingType = '';
        $shippingTitle = '';

        $session_shipping_method = WC()->session->get( 'chosen_shipping_methods' );
        if($session_shipping_method){
            $shipping_method = WC()->shipping->get_shipping_methods();

            $sessionMethods = substr($session_shipping_method[0],0,strrpos($session_shipping_method[0],':'));//Remove after colon text __:01, __:ex

            $shippingType = $shipping_method[$sessionMethods]->id;
            $shippingTitle = $shipping_method[$sessionMethods]->method_title;
        }

        $spec_lines = self::get_spec_lines($cart->get_cart());

        $shipping = (float)$cart->shipping_total;
        $shipping_tax = (float)$cart->shipping_tax_total;
        $shipping_total = (float)($shipping + $shipping_tax);

        /*
         * Compatibility.
         */
        $shipping_tax_pct = (!is_nan(@round($shipping_tax / $shipping, 2) * 100) ? @round($shipping_tax / $shipping,
                2) * 100 : 0);

        $spec_lines[] = [
            'id' => 'frakt',
            'artNo' => '00_frakt',
            'description' => $shippingType,
            'quantity' => '1',
            'unitMeasure' => '',
            'unitAmountWithoutVat' => $shipping,
            'vatPct' => $shipping_tax_pct,
            'totalVatAmount' => $shipping_tax,
            'totalAmount' => $shipping_total,
            'type' => 'SHIPPING_FEE',
        ];
        $payment_method = $woocommerce->session->chosen_payment_method;
        $payment_options = get_option('woocommerce_' . $payment_method . '_settings');

        $ResursFeeName = "";
        $fees = $cart->get_fees();
        if (is_array($fees)) {
            /** @var $fee WC_Cart_Fees */
            foreach ($fees as $fee) {
                if ($fee->tax > 0) {
                    $rate = ($fee->tax / $fee->amount) * 100;
                } else {
                    $rate = 0;
                }
                if (!empty($fee->id) && ($fee->amount > 0 || $fee->amount < 0)) {
                    $spec_lines[] = [
                        'id' => $fee->id,
                        'artNo' => $fee->id,
                        'description' => $fee->name,
                        'quantity' => 1,
                        'unitMeasure' => '',
                        'unitAmountWithoutVat' => $fee->amount,
                        'vatPct' => !is_nan($rate) ? $rate : 0,
                        'totalVatAmount' => $fee->tax,
                        'totalAmount' => $fee->amount + $fee->tax,
                        'type' => 'ORDER_LINE',
                    ];
                }
            }
        }
        if (wc_coupons_enabled()) {
            $coupons = $cart->get_coupons();
            if (is_array($coupons) && count($coupons) > 0) {
                $coupon_values = $cart->coupon_discount_amounts;
                $coupon_tax_values = $cart->coupon_discount_tax_amounts;

                /**
                 * @var  $code
                 * @var  $coupon WC_Coupon
                 */
                foreach ($coupons as $code => $coupon) {
                    $post = get_post($coupon->get_id());
                    $couponId = $coupon->get_id();
                    $couponCode = $coupon->get_code();
                    $couponDescription = $post->post_excerpt;
                    if (empty($couponDescription)) {
                        $couponDescription = $couponCode . '_' . __('coupon',
                                MOREFLO_CHECKOUT);
                    }
                    $spec_lines[] = [
                        'id' => $couponId,
                        'artNo' => $couponCode,
                        'description' => $couponDescription,
                        'quantity' => 1,
                        'unitMeasure' => '',
                        'unitAmountWithoutVat' => (0 - (float)$coupon_values[$code]) + (0 - (float)$coupon_tax_values[$code]),
                        'vatPct' => 0,
                        'totalVatAmount' => 0,
                        'totalAmount' => (0 - (float)$coupon_values[$code]) + (0 - (float)$coupon_tax_values[$code]),
                        'type' => 'DISCOUNT',
                    ];
                }
            }
        }
        $ourPaymentSpecCalc = self::calculateSpecLineAmount($spec_lines);
        if (!$specLinesOnly) {
            $payment_spec = [
                'specLines' => $spec_lines,
                'totalAmount' => $ourPaymentSpecCalc['totalAmount'],
                'totalVatAmount' => $ourPaymentSpecCalc['totalVatAmount'],
            ];
        } else {
            return $spec_lines;
        }

        return $payment_spec;
    }

    protected static function calculateSpecLineAmount($specLine = [])
    {
        $setPaymentSpec = ['totalAmount' => 0, 'totalVatAmount' => 0]; // defaults
        if (is_array($specLine) && count($specLine)) {
            foreach ($specLine as $row) {
                $setPaymentSpec['totalAmount'] += $row['totalAmount'];
                $setPaymentSpec['totalVatAmount'] += $row['totalVatAmount'];
            }
        }

        return $setPaymentSpec;
    }

    protected static function get_spec_lines($cart)
    {
        $spec_lines = [];
        foreach ($cart as $item) {
            /** @var WC_Product $data */
            $data = $item['data'];
            /** @var WC_Tax $_tax */
            $_tax = new WC_Tax();  //looking for appropriate vat for specific product
            $rates = [];
            $taxClass = $data->get_tax_class();
            $ratesArray = $_tax->get_rates($taxClass);
            $rates = @array_shift($ratesArray);
            if (isset($rates['rate'])) {
                $vatPct = (double)$rates['rate'];
            } else {
                $vatPct = 0;
            }

            $priceExTax = (!(self::version_check()) ? $data->get_price_excluding_tax() : wc_get_price_excluding_tax($data));
            $totalVatAmount = ($priceExTax * ($vatPct / 100));
            $bookArtId = $data->get_id();
            $postTitle = $data->get_title();

            $artDescription = (empty($postTitle) ? __('Article description missing',
                MOREFLO_CHECKOUT) : $postTitle);
            $spec_lines[] = [
                'id' => $bookArtId,
                'artNo' => $bookArtId,
                'description' => $artDescription,
                'quantity' => $item['quantity'],
                'unitMeasure' => '',
                'unitAmountWithoutVat' => $priceExTax,
                'vatPct' => $vatPct,
                'totalVatAmount' => ($priceExTax * ($vatPct / 100)),
                'totalAmount' => (($priceExTax * $item['quantity']) + ($totalVatAmount * $item['quantity'])),
                'type' => 'ORDER_LINE',
            ];
        }

        return $spec_lines;
    }

    private function createReturnUrls()
    {
        $returnArray = [];
        try {
            $successURL = $this->createResursOmniSuccessUrl(true);
            $failURL = $this->createResursOmniSuccessUrl();
            $returnArray = [
                'successUrl' => $successURL,
                'backUrl' => $failURL,
            ];
        } catch (Exception $e) {

        }

        return $returnArray;
    }

    private function createResursOmniSuccessUrl($isSuccess = false)
    {
        $this->omniSuccessUrl = get_site_url().'/';

        $omniRef = WC()->session->get(OMNIREF);

        $this->omniSuccessUrl = add_query_arg('wc-api', 'Invoice_Pay', $this->omniSuccessUrl);
        $this->omniSuccessUrl = add_query_arg('event-type', 'iframe_checkout', $this->omniSuccessUrl);
        $this->omniSuccessUrl = add_query_arg('payment_id', $omniRef, $this->omniSuccessUrl);
        $this->omniSuccessUrl = add_query_arg('set-no-session', '1', $this->omniSuccessUrl);
        $this->omniSuccessUrl = add_query_arg('success', $isSuccess ? 1 : 0, $this->omniSuccessUrl);

        $this->omniSuccessUrl = add_query_arg('omnicheckout_callback_mode', wp_create_nonce('omnicheckout_callback_nonce'), $this->omniSuccessUrl);


        return $this->omniSuccessUrl;
    }

    protected static function version_check( $version = '3.0' ) {
        if ( class_exists( 'WooCommerce' ) ) {
            global $woocommerce;
            if ( version_compare( $woocommerce->version, $version, ">=" ) ) {
                return true;
            }
        }
        return false;
    }

    public function resurs_callback_response()
    {
        global $wpdb;

        $mySession = false;
        $url_arr = parse_url($_SERVER["REQUEST_URI"]);
        $url_arr['query'] = str_replace('amp;', '', $url_arr['query']);
        parse_str($url_arr['query'], $request);
        if (!is_array($request)) {
            $request = [];
        }
        if (!count($request) && isset($_GET['event-type'])) {
            $request = $_GET;
        }
        $event_type = $request['event-type'];

        if ($event_type === 'fallPayment') {
            $this->reset_sessions();
        }

        if ($event_type === 'iframe_checkout') {
            $this->iframe_checkout();
        }

        /**Validate the callback was related this site event */
        $currentValidation = $this->validateCallback($request);

        If($currentValidation){

            if ($event_type != 'iframe_checkout') {
                $this->callbackevents($_REQUEST);
            }
        }
    }
	/**
	 * Payment form on checkout page.
	 */
	public function payment_fields() {
        if ( $this->description ) {
            echo wpautop( wp_kses_post( $this->description ) );
        }
    }
	/**
	 * Process the payment.
	 *
	 * @param int $order_id
	 */
	public function process_payment( $order_id ) {

        global $woocommerce;
        $order = new WC_Order( $order_id );

        $omniRef = WC()->session->get(OMNIREF);

        update_post_meta($order_id, 'paymentId', $omniRef);//Add payment ID asap in post meta table
        $this->write_Woo_logs('Processing order details', 'Order-ID#'.$order_id.' & resurs paymentId#'.$omniRef);
        // Return thankyou redirect
        return array(
            'result' => 'success'
        );

    }

    /**
     * Check result of Iframe checkout, book the payment and complete the order
     */
    public function iframe_checkout()
    {

        /** @var WC_Checkout $resursOrder What will be created if successful, and what will report undefined variable if unsuccessful */
        $resursOrder = null;

        // Get incoming request
        $url_arr = parse_url($_SERVER["REQUEST_URI"]);
        $url_arr['query'] = str_replace('amp;', '', $url_arr['query']);
        parse_str($url_arr['query'], $request);

        /*
            * Check the order reference against the session
            */
        $requestedPaymentId = WC()->session->get(OMNIREF);
        $requestedUpdateOrder = WC()->session->get('omniId');

        $orderID = $this->wc_get_order_id_by_payment_id($request['payment_id']);

        $resursPaymentObj;
        try {
            $resursPaymentObj = $this->auth->getPayment($request['payment_id']);
        } catch (Exception $e) {

        }
        $order = new WC_Order($orderID);

        if(!$this->validate_hasCall_order_id($orderID)){
            if($request['success'] == 1
            && $resursPaymentObj->status == 'IS_DEBITED' ){

                $this->appoinment_status($orderID, 'paid');
                $order->payment_complete();
                WC()->cart->empty_cart();
                $thankyouURL = $order->get_checkout_order_received_url();
                wp_safe_redirect($thankyouURL);
                $this->write_Woo_logs('Order response', 'Order success & IS_DEBITED Order-ID#'.$orderID);

            }elseif($request['success'] == 1
            && $resursPaymentObj->status == 'DEBITABLE'){

                $this->appoinment_status($orderID, 'paid');
                $order->payment_complete();
                WC()->cart->empty_cart();
                $thankyouURL = $order->get_checkout_order_received_url();
                wp_safe_redirect($thankyouURL);

            $this->write_Woo_logs('Order response', 'Order success & DEBITABLE  Order-ID#'.$orderID);

            }elseif($resursPaymentDetails->frozen != true
            && $resursPaymentDetails->fraud = true
            && $request['success'] == 1){

                $this->appoinment_status($orderID, 'pending-confirmation');
                $order->update_status( WC_PENDING_STATUS );
                WC()->cart->empty_cart();
                $thankyouURL = $order->get_checkout_order_received_url();
                wp_safe_redirect($thankyouURL);
                $this->write_Woo_logs('Order response', 'Order wc-pending Order-ID#'.$orderID);

            }elseif($resursPaymentDetails->frozen = true
            && $resursPaymentDetails->fraud != true
            && $request['success'] == 1){

                $this->appoinment_status($orderID, 'pending-confirmation');
                $order->update_status( WC_PENDING_STATUS );
                WC()->cart->empty_cart();
                $thankyouURL = $order->get_checkout_order_received_url();
                wp_safe_redirect($thankyouURL);
                $this->write_Woo_logs('Order response', 'Order wc-pending Order-ID#'.$orderID);

            }elseif( $request['success'] == 0){

                $this->appoinment_status($orderID, 'cancelled');
                $order->update_status( WC_CANCELLED_STATUS );
                wc_add_notice( __( 'Payment failed. Please contact bank for information', MOREFLO_CHECKOUT ), 'error' );
                wp_safe_redirect(wc_get_checkout_url());
                $this->write_Woo_logs('Order response', 'Order cancelled Order-ID#'.$orderID);

            }else{

                $this->appoinment_status($orderID, 'cancelled');
                $order->update_status( WC_CANCELLED_STATUS );
                wc_add_notice( __( 'Payment failed. Please contact bank for information', MOREFLO_CHECKOUT ), 'error' );
                wp_safe_redirect(wc_get_checkout_url());
                $this->write_Woo_logs('Order response', 'Order cancelled Order-ID#'.$orderID);

            }
        }else{

            WC()->cart->empty_cart();
            wc_add_notice( __( 'This is a already recorded payment.', MOREFLO_CHECKOUT ), 'error' );
            wp_safe_redirect(wc_get_checkout_url());

        }

        //Add hasCall meta key with payment id for which is stop runing next time
        update_post_meta($orderID, 'hasCall', $request['payment_id']);

        $this->resetSession();

        $this->init_session_variables();
    }

    public function reset_sessions(){

        $orderID = $this->wc_get_order_id_by_payment_id($_GET['payment_id']);

        $order = new WC_Order($orderID);
        $this->appoinment_status($orderID, 'cancelled');
        $order->update_status( WC_CANCELLED_STATUS );

        update_post_meta($orderID, 'hasCall', $_GET['payment_id']);

        $this->resetSession();

        $this->init_session_variables();

        $returnResult['success'] = true;
        $returnResult['errorString'] = __( 'Denied by Resurs', MOREFLO_CHECKOUT );
        $returnResult['errorCode'] = "200";
        $returnResult['newIframe'] = $this->resurs_omnicheckout_create_frame();

        wp_send_json( $returnResult, $returnResult['errorCode'] );
        wp_die();
    }

    public function appoinment_status($orderID, $new_status){

        if(class_exists('WC_Appointment_Data_Store')){
        $appointment_id = WC_Appointment_Data_Store::get_appointment_ids_from_order_id( $orderID );

        if($appointment_id)
        {
        $appointment = get_wc_appointment( $appointment_id[0] );
        $appointment->set_status( $new_status );
        $appointment->save();
        }
        }

    }

    public function callbackevents($request)
    {
        global $woocommerce;

        /** @var WC_Checkout $resursOrder What will be created if successful, and what will report undefined variable if unsuccessful */
        $resursOrder = null;

        $eventType = $request['event-type'];
        $returnResult = [];

        $orderId = $this->wc_get_order_id_by_payment_id($request['paymentId']);

        $resursPaymentObj = $this->auth->getPayment($request['paymentId']);

        $this->write_Woo_logs('Call back Event', $eventType);
        $this->write_Woo_logs('Call back payment obj', $resursPaymentObj);
        $this->write_Woo_logs('Call back status', $resursPaymentObj->status);
        if($resursPaymentObj->id){
            $this->write_Woo_logs('$resursPaymentObj->id', $resursPaymentObj->id);
        }

        $order = new WC_Order($orderId);

        if(is_array($resursPaymentObj->status)){//This is instamce update case
            if($resursPaymentObj->status[1] != 'IS_DEBITED'){
                switch ($eventType) {
                    case 'UNFREEZE':
                        $order->update_status( WC_PENDING_STATUS );
                        $this->appoinment_status($orderId, 'pending-confirmation');
                        $returnResult['success'] = true;
                        $returnResult['successCode'] = "204";

                        $this->write_Woo_logs('Order Status', WC_PENDING_STATUS);

                        break;
                    case 'ANNULMENT':
                        $order->update_status( WC_CANCELLED_STATUS );
                        $this->appoinment_status($orderId, 'cancelled');
                        $returnResult['success'] = true;
                        $returnResult['successCode'] = "204";

                        $this->write_Woo_logs('Order Status', WC_CANCELLED_STATUS);

                        break;
                }
            }else{
                if($resursPaymentObj->id){
                    $order->set_transaction_id($resursPaymentObj->id);
                }
                $order->payment_complete();
                $this->appoinment_status($orderId, 'paid');
                $returnResult['success'] = true;
                $returnResult['successCode'] = "204";

                $this->write_Woo_logs('Order Status', WC_COMPLETED_STATUS);
            }
        }else{
            if($resursPaymentObj->status != 'DEBITABLE'){
                switch ($eventType) {
                    case 'UNFREEZE':
                        $order->update_status( WC_PENDING_STATUS );
                        $this->appoinment_status($orderId, 'pending-confirmation');
                        $returnResult['success'] = true;
                        $returnResult['successCode'] = "204";

                        $this->write_Woo_logs('Order Status', WC_PENDING_STATUS);

                        break;
                    case 'ANNULMENT':
                        $order->update_status( WC_CANCELLED_STATUS );
                        $this->appoinment_status($orderId, 'cancelled');
                        $returnResult['success'] = true;
                        $returnResult['successCode'] = "204";

                        $this->write_Woo_logs('Order Status', WC_CANCELLED_STATUS);

                        break;
                }
            }else{
                if($resursPaymentObj->id){
                    $order->set_transaction_id($resursPaymentObj->id);
                }
                $order->payment_complete();
                $this->appoinment_status($orderId, 'paid');
                $returnResult['success'] = true;
                $returnResult['successCode'] = "204";

                $this->write_Woo_logs('Order Status', WC_COMPLETED_STATUS);
            }
        }


        wp_send_json( $returnResult, $returnResult['successCode'] );
    }

    public function createManualOrderForCallbacks($createPayRequestObj)
    {
        global $woocommerce;

        try {
            $resursPaymentDetails = $this->auth->getPayment($createPayRequestObj['paymentId']);
        } catch (Exception $e) {

        }

        $this->write_Woo_logs('Callbacks', $createPayRequestObj);

        $orderId = $this->createManualOrder($resursPaymentDetails, $createPayRequestObj, $redirect = false);
        return $orderId;

    }

    function wc_get_order_id_by_payment_id($paymentId = '')
    {
        global $wpdb;
        $order_id = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = 'paymentId' and meta_value = '%s'",
                $paymentId
            )
        );

        return $order_id;
    }

    function wc_get_payment_id_by_order_id($orderId = '')
    {
        global $wpdb;
        $order_id = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT meta_value FROM {$wpdb->prefix}postmeta WHERE meta_key = 'paymentId' and post_id = '%s'",
                $orderId
            )
        );

        return $order_id;
    }

    function validate_hasCall_order_id($orderId = '')
    {
        global $wpdb;
        $order_id = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT meta_value FROM {$wpdb->prefix}postmeta WHERE meta_key = 'hasCall' and post_id = '%s'",
                $orderId
            )
        );

        return $order_id;
    }



    public function getCallbackTypes()
    {
        return [
            'UNFREEZE' => [
                'uri_components' => [
                    'paymentId' => 'paymentId',
                ],
                'digest_parameters' => [
                    'paymentId' => 'paymentId',
                ],
            ],
            'ANNULMENT' => [
                'uri_components' => [
                    'paymentId' => 'paymentId',
                ],
                'digest_parameters' => [
                    'paymentId' => 'paymentId',
                ],
            ],
            'BOOKED' => [
                'uri_components' => [
                    'paymentId' => 'paymentId',
                ],
                'digest_parameters' => [
                    'paymentId' => 'paymentId',
                ],
            ],
            'UPDATE' => [
                'uri_components' => [
                    'paymentId' => 'paymentId',
                ],
                'digest_parameters' => [
                    'paymentId' => 'paymentId',
                ],
            ],
        ];
    }

    public function updateRegisterCallbacks()
    {
        foreach ($this->callback_types as $callback => $options) {
            $createPayRequestObj = $this->registerCustomCallBacks($callback, $options);
        }
    }

    public function registerCustomCallBacks($type, $options)
    {
        $uriTemplate = null;
        try {
            $homeURL = get_site_url().'/';
            $uriTemplate = $homeURL;
            $uriTemplate = add_query_arg('wc-api', 'Invoice_Pay', $uriTemplate);
            $uriTemplate .= '&event-type=' . $type;
            foreach ($options['uri_components'] as $key => $value) {
                $uriTemplate .= '&' . $key . '=' . '{' . $value . '}';
            }
            $uriTemplate .= '&digest={digest}';

            $callbackType = $this->auth->getCallbackTypeByString($type);
            $salt = get_option('salt_key');
            if(!$salt){
                $salt = uniqid(mt_rand(), true);
                add_option('salt_key', $salt);
            }
            $this->auth->setCallbackDigestSalt($salt);
            $this->auth->setRegisterCallback($callbackType, $uriTemplate);
        } catch (Exception $e) {
            throw new Exception($e);
        }

        return $uriTemplate;
    }

    function getUserIpAddr(){
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP'))
            $ipaddress = getenv('HTTP_CLIENT_IP');
        else if(getenv('HTTP_X_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        else if(getenv('HTTP_X_FORWARDED'))
            $ipaddress = getenv('HTTP_X_FORWARDED');
        else if(getenv('HTTP_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        else if(getenv('HTTP_FORWARDED'))
            $ipaddress = getenv('HTTP_FORWARDED');
        else if(getenv('REMOTE_ADDR'))
            $ipaddress = getenv('REMOTE_ADDR');
        else
            $ipaddress = 'UNKNOWN';

        return $ipaddress;
    }


    private function validateCallback($request)
    {
        $success = null;

        $paymentId = isset($request['paymentId']) ? $request['paymentId'] : null;
        $digest = isset($request['digest']) ? $request['digest'] : null;
        $result = isset($request['result']) ? $request['result'] : null;

        $testDigestArray = [$paymentId];

        foreach ($testDigestArray as $testId) {
            if ($this->getValidatedDigestResponse(
                $testId,
                get_option('salt_key'),
                $digest,
                $result)
            ) {
                $success = $testId;
                break;
            }
        }

        return (string)$success;
    }

    private function getValidatedDigestResponse($paymentId, $currentSalt, $digest, $result)
    {
        return $this->auth->getValidatedCallbackDigest($paymentId, $currentSalt, $digest, $result);
    }

    /**
     * Return json response received notification
     */
    private function returnJsonResponse($jsonArray = [], $responseCode = 200)
    {
        header("Content-Type: application/json", true, $responseCode);
        echo json_encode($jsonArray);
        wp_die();
    }

    /**
     * Written log in the debug log file
     * Ex: $this->write_logs_site($details);
     */
    public function write_logs_site($details)
    {
        //$log = 'write this line';
        if (!function_exists('write_log')) {
            function write_log ( $log )  {
                if ( true === WP_DEBUG ) {
                if ( is_array( $log ) || is_object( $log ) ) {
                    error_log( print_r( $log, true ) );
                } else {
                    error_log( $log );
                }
                }
            }
        }
        write_log($details);
    }

    /**
     * Write logs in the woocommerce debug log
     * Ex: $this->write_Woo_logs($name, $details);
     */
    public function write_Woo_logs($name, $details)
    {
        if ( $this->log_enabled == 'yes' ) {//record if only log enable this method to run
            $this->logger->debug(wc_print_r("******".$name."******", true), $this->logger_context);
            $this->logger->debug(wc_print_r($details, true), $this->logger_context);
            $this->logger->debug(wc_print_r("******".$name."-End******", true), $this->logger_context);
        }
    }
}

add_action( 'woocommerce_cart_updated', 'Invoice_Pay::resurs_omnicheckout_update' );
