<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Pay with Morflo Invoice
 *
 * @class 		Moreflo_Invoice_Pay
 * @extends		WC_Payment_Gateway
 * @version		1.0.0
 */

class Moreflo_Invoice_Pay extends WC_Payment_Gateway {

    public function __construct() {
        $this->id                 = 'moreflo_invoice';
		$this->method_title       = __( 'MoreFlo Checkout', 'moreflo-checkout' );
        $this->method_description = __( 'Use invoice payment.', 'moreflo-checkout' );
        $this->has_fields         = false;
        $this->supports           = array(
            'products'
        );

        $this->init_form_fields();
        $this->init_settings();
        $this->title = $this->get_option( 'title' );
        $this->description = $this->get_option( 'description' );

        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
    }

    public function init_form_fields() {

        $this->form_fields = array(
            'enabled' => array(
                'title' => __( 'Enable/Disable', 'moreflo-checkout' ),
                'type' => 'checkbox',
                'label' => __( 'Enable Invoice Payment', 'moreflo-checkout' ),
                'default' => 'no'
            ),
            'title' => array(
                'title' => __( 'Title', 'moreflo-checkout' ),
                'type' => 'text',
                'description' => __( 'This controls the title which the user sees during checkout within the invoice payment type.', 'moreflo-checkout' ),
                'default' => __( 'Pay with invoice', 'moreflo-checkout' ),
            ),
            'description' => array(
                'title' => __( 'Description', 'moreflo-checkout' ),
                'type' => 'text',
                'default' => __( 'Pay with invoice', 'moreflo-checkout' )
            )
        );
    }

    public function process_payment( $order_id ) {

        global $woocommerce;
        $order = new WC_Order( $order_id );

        $order->update_status('on-hold', __( 'Awaiting invoice payment', 'moreflo-checkout' ));
        $woocommerce->cart->empty_cart();

        return array(
            'result' => 'success',
            'redirect' => $this->get_return_url( $order )
        );
    }

    public function payment_fields() {
        ob_start();
        if ( $this->description ) {
            echo wpautop( wp_kses_post( $this->description ) );
        }

        $fields_html = ob_get_clean();

        echo apply_filters( 'moreflo_checkout_moreflo_invoice_pay_payment_fields', $fields_html );
    }

    public function payment_scripts() {

    }

    public function validate_fields() {

	}
}
