<?php
class Moreflo_swish_client extends Swish_Pay_Gateway{

    public function preparePaymentClient($order, $payer, $merchant)
    {

        $order_id = $order->get_id();

        $secret = $this->get_secret($order_id);
        if (!$secret) {
            $secret = uniqid();
            $this->set_secret($order_id, $secret);
        }

        $transaction_textarray = array();
        if (property_exists( $this, 'text_on_transaction' ) && $this->text_on_transaction != '') {
            $transaction_textarray[] = $this->text_on_transaction;
        }

        $customer_number = $order->get_user_id();

        $items = $order->get_items();

        $product_list = [];
        foreach ( $items as $item ) {
            array_push($product_list, $item->get_name());
        }

        $products = implode(",",$product_list);

        if($customer_number){
            $transaction_textarray[] = sprintf(__('Customer number %s', 'moreflow-pay'), $customer_number);
        }else{
            $transaction_textarray[] = $products;
        }

        $transaction_text = mb_substr(preg_replace("/[^a-zA-Z0-9åäöÅÄÖ :;.,?!()]+/u", "", implode(', ', $transaction_textarray)), 0, 49);

        $payment = array(
            'payeePaymentReference' => $order->get_order_number(),
            'callbackUrl' => $this->get_callback_url($order_id, $secret, $merchant, 'payment'),
            'payerAlias' => $payer,
            'payeeAlias' => $merchant,
            'amount' => str_replace(',', '.', $order->get_total()),
            //'currency' => 'SEK',
            'currency' => $order->get_currency(),
            'message' => $transaction_text,
        );
        return $payment;
    }
    public function prepareRefundPaymentClient($transaction_id, $order, $amount, $reason, $merchant)
    {
        $order_id = $order->get_id();

        $secret = $this->get_secret($order_id);
        if (!$secret) {
            $secret = uniqid();
            $this->set_secret($order_id, $secret);
        }

        if ($amount === null) {
            $amount = $order->get_total();
        }

        $transaction_text = mb_substr(preg_replace("/[^a-zA-Z0-9åäöÅÄÖ :;.,?!()]+/u", "", $reason), 0, 49);

        $refundpayment = array(
            'payerPaymentReference' => 'R-' . $order_id . '-' . date('Y-m-d H:i', current_time('timestamp')),
            'originalPaymentReference' => $transaction_id,
            'callbackUrl' => $this->get_callback_url($order_id, $secret, $merchant, 'refund'),
            'payerAlias' => $merchant,
            'amount' => str_replace(',', '.', $amount),
            //'currency' => 'SEK',
            'currency' => $order->get_currency(),
            'message' => $transaction_text,
        );

        return $refundpayment;
    }

    public function get_secret($order_id)
    {
        return get_post_meta($order_id, '_swish_unique_id', true);
    }

    public function set_secret($order_id, $secret)
    {
        update_post_meta($order_id, '_swish_unique_id', $secret);
    }

    public function get_callback_url($order_id, $secret, $merchant, $type)
    {
        $security_key = $this->security_code($secret, $order_id, $merchant);
        $args = array(
            'wc-api' => 'swish_pay_gateway',
            'event' => $type,
            'nonce' => $security_key,
            'order_id' => $order_id,
        );

        return add_query_arg($args, site_url('/'));

    }

    public function verify_callback($received_nonce, $order_id, $merchant)
    {
        $secret = $this->get_secret($order_id);

        $security_key = $this->security_code($secret, $order_id, $merchant);

        //add this logic( substr($security_key, 0, -1)) which is remove added extra charactor
        if(substr($security_key, 0, -1) == $received_nonce){
            return true;
        }else{
            return false;
        }
    }

    public function security_code($secret, $order_id, $merchant){
        return $security_key = base64_encode(sha1($secret.$order_id.$merchant));
    }
}

?>
