<?php

require_once DIRPATH.'classes/api/class-woocommerce-swish-pay-error.php';

class Moreflo_swish_api extends Moreflo_swish_errors
{
    
    public $api_url;
    private $root_certificate;
    private $public_key_certificate;
    private $private_key_certificate;
    private $private_key_password;
    private $merchant_alias;

    public function __construct( $url, $merchant, $root_certificate, $public_certificate, $private_certificate, $private_key_password, $logEnabled ) {

        $this->api_url                 = $url;
        $this->root_certificate        = $root_certificate;
        $this->public_key_certificate  = $public_certificate;
        $this->private_key_certificate = $private_certificate;
        $this->private_key_password    = $private_key_password;
        $this->merchant_alias          = $merchant;
        $this->logger                  = wc_get_logger();
        $this->log_enabled             = $logEnabled;
        $this->logger_context          = array( 'source' => 'swish_pay_gateway' );
    }

    public function endpoint( $path ) {
        return 'https://' . $this->api_url . '/swish-cpcapi/api/v1/' . $path;
    }

    public function apiRequest( $payer_alias, $path, $type = 'POST' ) {
        
        $end_point = $this->endpoint( $path );
        $data_string = json_encode( $payer_alias, JSON_UNESCAPED_SLASHES );

	    return $this->curlObjPost( $end_point, $data_string, $type );
    }

    public function curlObjPost( $end_point, $data_string, $type ) {
	    $extension = strtoupper( pathinfo( $this->public_key_certificate, PATHINFO_EXTENSION ) );

		$cert_check_valid = $this->verify_certificate(
			$this->root_certificate,
			$this->public_key_certificate,
			$this->private_key_certificate,
			$extension
		);


		if ( true !== $cert_check_valid ) {
			$this->write_Woo_logs_2( $cert_check_valid );
			return array( 'status' => false, 'response' => __( 'Could not connect to Swish.', 'moreflo-checkout' ), 'response_technical' => $cert_check_valid );
		}

        $ch = curl_init();

        curl_setopt( $ch, CURLOPT_HEADER, true );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
        
        curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 2 );
        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );
        curl_setopt( $ch, CURLINFO_HEADER_OUT, true );

        curl_setopt( $ch, CURLOPT_CAINFO, $this->root_certificate );
        curl_setopt( $ch, CURLOPT_SSLCERT, $this->public_key_certificate );
        curl_setopt( $ch, CURLOPT_URL, $end_point );
        curl_setopt( $ch, CURLOPT_CUSTOMREQUEST, $type );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $data_string );

	    if ( 'P12' === $extension ) {
		    curl_setopt( $ch, CURLOPT_SSLCERTTYPE, "P12" );
		    curl_setopt( $ch, CURLOPT_SSLKEYPASSWD, $this->private_key_password );

            if ( ! $this->private_key_password ) {
                $this->write_Woo_logs_2( 'Private key not set.' );
            }
	    } elseif ( 'PEM' === $extension ) {
		    curl_setopt( $ch, CURLOPT_SSLKEY, $this->private_key_certificate );
			if ( $this->private_key_password ) {
		        curl_setopt( $ch, CURLOPT_SSLKEYPASSWD, $this->private_key_password );
			}
	    }

		if ( 'POST' === $type ) {
			curl_setopt( $ch, CURLOPT_POST, true );
		}

		$content_type = 'application/json';
		if ( 'PATCH' === $type ) {
			$content_type = 'application/json-patch+json';
		}

		curl_setopt(
			$ch,
		    CURLOPT_HTTPHEADER,
		    array(
		        'Content-Type: ' . $content_type,
		        'Content-Length: ' . strlen( $data_string ),
		    )
        );

        $curl_response  = curl_exec( $ch );
        $curl_error     = curl_error( $ch );
        $response_code  = (int) curl_getinfo( $ch, CURLINFO_HTTP_CODE );

        $header_size      = curl_getinfo( $ch, CURLINFO_HEADER_SIZE );
        $result['header'] = substr( $curl_response, 0, $header_size );
        $result['body']   = substr( $curl_response, $header_size );

        curl_close( $ch );

        if ( $curl_response ){
            if ( $response_code >= 200 && $response_code <= 299 ){
                preg_match( '/Location:(.*?)\n/', $result['header'], $matches );
                $location = trim( array_pop( $matches ) );
                $this->write_Woo_logs_2( $location );
                $return_result = array( 'status' => true, 'response' => $location, 'content' => $curl_response );
            } elseif ( $response_code >= 400 && $response_code <= 499 ) {
                $this->write_Woo_logs_2( $result );
                $errors = json_decode( $result['body'] );

				if( is_array( $errors ) ){
					$errors = $errors[0];
				}

				if ( isset( $errors->errorCode ) ) {
					$error_details = ( $errors->errorCode )
						? $this->error_code( $errors->errorCode )
						: __( 'Could not connect to Swish.', 'moreflo-checkout' );
					$error_technical = $error_details;
				} else {
					$error_details   = __( 'Could not connect to Swish.', 'moreflo-checkout' );
					$error_technical = $errors->error ?? $error_details;
				}

                $this->write_Woo_logs_2( 'Error during call: '. $error_technical );
                $this->write_Woo_logs_2( 'Response code: ' . $response_code );
                $return_result = array( 'status' => false, 'response' => $error_details, 'response_technical' => $error_technical );
            } else {
                $this->write_Woo_logs_2( 'Error during call: Unknown response code ' . $response_code );
                $return_result = array( 'status' => false, 'response' => __( 'Could not connect to Swish.', 'moreflo-checkout' ), 'response_technical' => 'Unknown response code' );
            }
        } else {
			$this->write_Woo_logs_2( 'Error during call: '. $curl_error );
            $return_result = array( 'status' => false, 'response' => __( 'Could not connect to Swish.', 'moreflo-checkout' ), 'response_technical' => $curl_error );
			$this->write_Woo_logs_2( 'Root cert: '. $this->root_certificate );
			$this->write_Woo_logs_2( 'Public cert: '. $this->public_key_certificate );
			$this->write_Woo_logs_2( 'Private cert: '. $this->private_key_certificate );
        }
        return $return_result;
    }


	private function verify_certificate( $root_cert, $public_cert, $private_cert, $extension ) {

		if ( ! file_exists( $root_cert ) ) {
			return 'Root certificate not found: ' . $root_cert;

		} elseif ( ! file_exists( $public_cert ) ) {
			return 'Public certificate not found: ' . $public_cert;

		} else {
		    if ( 'P12' === $extension ) {
			    $p12file           = file_get_contents( $public_cert ); // Public and private key in one file.
			    $certs             = array();
			    $cert_parse_result = openssl_pkcs12_read( $p12file, $certs, $this->private_key_password );
				if ( ! $cert_parse_result ) {
					return 'Certificate parse failed with error: ' . openssl_error_string();
				} else {
					return true;
				}

		    } elseif ( 'PEM' === $extension ) {
			    if ( ! file_exists( $private_cert ) ) {
				    return 'Private key not found: ' . $private_cert;
			    }

				return true;
				// Test gives false fails.

				$data = 'Test data';
			    openssl_sign( $data, $signature, file_get_contents( $private_cert ), "sha256WithRSAEncryption" );
			    $ok = openssl_verify($data, $signature, file_get_contents( $public_cert ) );

			    if ( 1 == $ok ) {
				    return true;
			    } elseif ($ok == 0) {
				    return 'Certificates invalid';
			    } else {
				    return "Certificate parse failed with error: " . openssl_error_string();
			    }
			} else {
				return 'Unknown certificate extension.' . print_r( pathinfo( $this->public_key_certificate ), true );
		    }
		}
	}

    public function write_Woo_logs_2( $details ) {
        if ( $this->log_enabled == 'yes' ) {
            $this->logger->debug( wc_print_r( $details, true ), $this->logger_context );
        }
    }
}
