<?php
class MFPay_update {
	public function __construct(){
		add_filter(
			'plugins_api',
			array(
                &$this,
                'mf_plugin_info'
            ),
			20,
			3
		);

		add_filter(
			'site_transient_update_plugins',
			array(
                &$this,
                'mf_push_updates'
			)
		);

		add_filter(
			'upgrader_process_complete',
			array(
                &$this,
                'mf_after_update'
            ),
			10,
			2
		);
	}

	public function mf_plugin_info( $res, $action, $args ) {
		if ( 'plugin_information' !== $action || 'moreflo-checkout' !== $args->slug ) {
			return $res;
		}

		if ( get_transient( 'moreflo_pay_upgrade_error' ) ) {
			return $res;
		}

		$remote = $this->get_remote_plugin_info();

		if ( $remote ) {
			$remote              = json_decode($remote['body']);
			$res                 = new stdClass();
			$res->name           = $remote->name;
			$res->slug           = 'moreflo-checkout';
			$res->version        = $remote->version;
			$res->tested         = $remote->tested;
			$res->requires       = $remote->requires;
			$res->author         = '<a href="https://www.northmill.com/se/foretag/">Northmill Flo</a>';
			$res->author_profile = 'https://www.northmill.com/se/foretag/';
			$res->download_link  = $remote->download_url;
			$res->trunk          = $remote->download_url;
			$res->last_updated   = $remote->last_updated;
			$res->sections       = array(
				'changelog' => $remote->sections->changelog
			);
			if ( ! empty( $remote->sections->screenshots ) ) {
				$res->sections['screenshots'] = $remote->sections->screenshots;
			}
			$res->banners = array(
				'low' => 'https://downloadsflo.northmill.com/wp-content/uploads/2017/04/moreflo-system-1024x532.png',
				'high' => 'https://downloadsflo.northmill.com/wp-content/uploads/2017/04/moreflo-system-1024x532.png'
			);
		}

		return $res;
	}

	public function mf_push_updates( $transient ) {
		if ( empty( $transient->checked ) ) {
			return $transient;
		}

		if ( get_transient( 'moreflo_pay_upgrade_error' ) ) {
			return $transient;
		}

		$remote = $this->get_remote_plugin_info();

		$plugin_data    = get_plugin_data(plugin_dir_path(__FILE__) . '/moreflo-checkout.php');
		$plugin_version = $plugin_data['Version'];

		if ( $remote ) {
			$remote = json_decode($remote['body']);
			if (
				$remote &&
				version_compare( $plugin_version, $remote->version, '<' ) &&
				version_compare( $remote->requires, get_bloginfo('version'), '<' )
			) {
				$res                               = new stdClass();
				$res->slug                         = 'moreflo-checkout';
				$res->plugin                       = plugin_basename(__DIR__) . '/moreflo-checkout.php';
				$res->new_version                  = $remote->version;
				$res->tested                       = $remote->tested;
				$res->package                      = $remote->download_url;
				$res->compatibility                = new stdClass();
				$transient->response[$res->plugin] = $res;
			}
		}

		return $transient;
	}

	/**
	 * Get plugin info from Northmill server.
	 *
	 * @return array|false
	 */
	private function get_remote_plugin_info() {
		$remote = get_transient( 'moreflo_pay_upgrade' );
		if ( ! $remote ) {
			$remote = wp_remote_get(
				'https://downloadsflo.northmill.com/plugin/moreflo-checkout.json',
				array(
					'timeout' => 10,
					'headers' => array(
						'Accept' => 'application/json',
					),
				)
			);

			if (
				! is_wp_error( $remote ) &&
				isset( $remote[ 'response' ]['code'] ) &&
				200 === $remote[ 'response' ]['code'] &&
				! empty( $remote['body'] )
			) {
				set_transient( 'moreflo_pay_upgrade', $remote, HOUR_IN_SECONDS * 12 ); // 12 hours cache
			} else {
				$remote = false;
				set_transient( 'moreflo_pay_upgrade_error', true, HOUR_IN_SECONDS );
			}
		}

		return $remote;
	}

	public function mf_after_update( $upgrader_object, $options ) {
		if ( 'update' === $options['action'] && 'plugin' === $options['type']) {
			delete_transient( 'moreflo_pay_upgrade' );
		}
	}
}
new MFPay_update();
