<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}
/**
 * Plugin Name: MoreFlo Checkout
 * Plugin URI: https://northmill.com/se/foretag/betallosningar/checkout/
 * Description: A payment gateway for WooCommerce. Credit card payment via Northmill Flo. Support for invoice, Swish and partial payment via Resurs Bank.
 * Author: Northmill Flo
 * Author URI: https://northmill.com/se/foretag/
 * Version: 1.7.18
 * WC tested up to:  4.5.1
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */
define('MOREFLO_CHECKOUT', 'moreflo-checkout');
define('WC_FAILED_STATUS', 'wc-failed');
define('WC_COMPLETED_STATUS', 'wc-completed');
define('WC_PENDING_STATUS', 'wc-pending');
define('WC_CANCELLED_STATUS', 'wc-cancelled');
define('DIRPATH', plugin_dir_path(__FILE__));

$file_data = get_file_data( __FILE__, array( 'Version' => 'Version' ) );
define( 'MOREFLO_CHECKOUT_VERSION', $file_data['Version'] );

if ( is_admin() ) {
    require_once plugin_dir_path(__FILE__) . 'moreflo-checkout-update-cc.php';
}
register_activation_hook(__FILE__, 'my_plugin_activate');
function my_plugin_activate(){
    if(!class_exists("SOAPClient")){ // Plugin validate soapclient enable or not.
        die('Plugin NOT activated, because SOAPClient is not activated your hosting !!');
    }
}
/**
 *  If WooCommerce is active add load up the gateway
 **/
// add_action( 'plugins_loaded', 'cybersource_credit_cards', 0 );
add_action( 'plugins_loaded', 'invoice_payment', 0 );
add_action( 'plugins_loaded', 'nmi_payment', 0 );
add_action( 'plugins_loaded', 'moreflo_invoice_payment', 0 );
add_action( 'plugins_loaded', 'swish_payment', 0 );


/**
 * Add the CyberSource Credit Card gateway class to the Woocommerce Gateway list
 *
 */
// function cybersource_credit_cards() {
// 	include_once('class-woocommerce-cybersource-cc.php');

// 	function add_cybersource_cc_class( $methods ) {
// 		$methods[] = 'CyberSource_CC_Gateway';
// 		return $methods;
// 	}
//     add_filter( 'woocommerce_payment_gateways', 'add_cybersource_cc_class' );
// }

function invoice_payment() {
	include_once('class-woocommerce-invoice-pay.php');

	function add_invoice_cc_class( $methods ) {
		$methods[] = 'Invoice_Pay';
		return $methods;
	}
    add_filter( 'woocommerce_payment_gateways', 'add_invoice_cc_class' );
}

function nmi_payment() {
	include_once('class-woocommerce-nmi-pay.php');

	function add_nmi_payment_class( $methods ) {
		$methods[] = 'NmiPayGateway';
		return $methods;
	}
    add_filter( 'woocommerce_payment_gateways', 'add_nmi_payment_class' );
}

function moreflo_invoice_payment(){
    include_once('class-woocommerce-moreflo-invoice-pay.php');

    add_action('add_meta_boxes', 'moreflo_invoice_pay_add_meta_box');

    function add_moreflo_invoice_payment_class( $methods ) {
        $methods[] = 'Moreflo_Invoice_Pay';
        return $methods;
    }
    add_filter( 'woocommerce_payment_gateways', 'add_moreflo_invoice_payment_class' );
}

function swish_payment() {
	include_once('class-woocommerce-swish-pay.php');

	function add_swish_payment_class( $methods ) {
		$methods[] = 'Swish_Pay_Gateway';
		return $methods;
	}
   add_filter( 'woocommerce_payment_gateways', 'add_swish_payment_class' );

   add_action( 'wp_ajax_nopriv_swish_pay_status', 'Swish_Pay_Gateway::return_swish_pay_status' );
   add_action( 'wp_ajax_swish_pay_status', 'Swish_Pay_Gateway::return_swish_pay_status' );
   add_action( 'wp_ajax_swish_pay_test_connection', 'Swish_Pay_Gateway::run_connection_test' );
   add_action( 'wp_ajax_swish_pay_abort_test_payment', 'Swish_Pay_Gateway::abort_connection_test_request' );
   add_action( 'wp_ajax_swish_pay_generate_new_certificate', 'Swish_Pay_Gateway::generate_new_certificate' );
   add_action( 'woocommerce_settings_checkout', 'Swish_Pay_Gateway::render_certificate_form_modal', 20 );
   add_action( 'admin_post_download_cert', 'Swish_Pay_Gateway::download_cert' );
}

function load_txtdomain() {
    load_plugin_textdomain(MOREFLO_CHECKOUT, false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}

add_action( 'plugins_loaded', 'load_txtdomain' );

if ( ! function_exists( 'moreflo_invoice_pay_add_meta_box' ) )
{
    function moreflo_invoice_pay_add_meta_box()
    {
        global $post;

        $wcorder = wc_get_order($post->ID);
        if( $wcorder instanceof WC_Order ){

            $order = new WC_Order($post->ID);
            $paymethod = $order->get_payment_method();
            if($paymethod == 'moreflo_invoice'){
            add_meta_box( 'moreflo_invoice_meta_box', __('MoreFlo Invoice Number','moreflo-checkout'), 'moreflo_invoice_pay_form', 'shop_order', 'side', 'core' );
            }
        }
    }
}

if ( ! function_exists( 'moreflo_invoice_pay_form' ) )
{
    function moreflo_invoice_pay_form()
    {
        global $post;

        $meta_field_data = get_post_meta( $post->ID, '_moreflo_invoice_number', true ) ? get_post_meta( $post->ID, '_moreflo_invoice_number', true ) : '';

        echo '<input type="hidden" name="moreflo_invoice_meta_field_nonce" value="' . wp_create_nonce() . '">
        <p style="border-bottom:solid 1px #eee;padding-bottom:13px;">
            <input type="text" style="width:250px;";" name="moreflo_invoice" placeholder="' . $meta_field_data . '" value="' . $meta_field_data . '"></p>';
    }
}

    add_action( 'save_post', 'save_moreflo_invoice_pay', 10, 1 );
if ( ! function_exists( 'save_moreflo_invoice_pay' ) )
    {
        function save_moreflo_invoice_pay( $post_id ) {

            if ( ! isset( $_POST[ 'moreflo_invoice_meta_field_nonce' ] ) ) {
                return $post_id;
            }
            $nonce = $_REQUEST[ 'moreflo_invoice_meta_field_nonce' ];

            if ( ! wp_verify_nonce( $nonce ) ) {
                return $post_id;
            }

            if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
                return $post_id;
            }

            if ( 'page' == $_POST[ 'post_type' ] ) {

                if ( ! current_user_can( 'edit_page', $post_id ) ) {
                    return $post_id;
                }
            } else {

                if ( ! current_user_can( 'edit_post', $post_id ) ) {
                    return $post_id;
                }
            }

            update_post_meta( $post_id, '_moreflo_invoice_number', $_POST[ 'moreflo_invoice' ] );
        }
    }

// add_action( 'wp', 'delete_payment_method_action', 0 );
/**
 *  This function will delete tokens at CyberSource when the token is deleted
 *  using the manage accounts functionality with Woocommerce.
 *  TODO: See if there is a more direct hook then the generic wp hook.
 */
function delete_payment_method_action() {
    global $wp;

    if ( isset( $wp->query_vars['delete-payment-method'] ) ) {

        $token_id = absint( $wp->query_vars['delete-payment-method'] );
        $token = WC_Payment_Tokens::get( $token_id );

        //error_log(print_r($token, true) . "\n\n");

        $delete = true;

        if ( is_null( $token ) ) {
            $delete = false;
        }

        if ( get_current_user_id() !== $token->get_user_id() ) {
            $delete = false;
        }

        if ( false === wp_verify_nonce( $_REQUEST['_wpnonce'], 'delete-payment-method-' . $token_id ) ) {
            $delete = false;
        }

        if ( $delete ) {
            $available_gateways = WC()->payment_gateways->get_available_payment_gateways();
            $result = $available_gateways[ 'cybersource_cc' ]->delete_payment_method($token->get_token());

            if ( $result['result'] == 'success' ) {
                wc_add_notice( __( 'Payment method deleted at CyberSource.', 'woocommerce' ) );
            }
        }
    }
}

function flo_checkout_payment_gateways_creating_receipts( $receipt_gateways ) {
	$receipt_gateways['invoice_pays']      = 'ResursBank';
	$receipt_gateways['nmi_pay_gateway']   = 'BankCardPayment';
	$receipt_gateways['swish_pay_gateway'] = 'Swish';
	return $receipt_gateways;
}
add_filter( 'flo_sync_payment_gateways_creating_receipts', 'flo_checkout_payment_gateways_creating_receipts' );
